# -*- coding: utf-8
import inspect
import threading

from django.conf import settings
from django.utils.module_loading import import_string
from optic import Optic

from .apps import OpticDjangoAppConfig

local = threading.local()


class BasicOpticManager:
    optic = None

    @classmethod
    def get_interaction_container(cls):
        return getattr(cls, "interaction_container", None)

    @classmethod
    def is_middleware_class(cls, middleware_path) -> bool:
        from optic_django_middleware.middleware import OpticMiddleware

        try:
            middleware_cls = import_string(middleware_path)
        except ImportError:
            return False
        return inspect.isclass(middleware_cls) and issubclass(
            middleware_cls, OpticMiddleware
        )

    @classmethod
    def add_middleware(cls):
        middleware_class_name = "optic_django_middleware.middleware.OpticMiddleware"
        middleware_setting = getattr(settings, "MIDDLEWARE", None)
        setting_name = "MIDDLEWARE"
        if middleware_setting is None:
            middleware_setting = settings.MIDDLEWARE_CLASSES
            setting_name = "MIDDLEWARE_CLASSES"

        # add the middleware only if it was not added before
        if not any(map(cls.is_middleware_class, middleware_setting)):
            if isinstance(middleware_setting, list):
                new_middleware_setting = middleware_setting + [middleware_class_name]
            elif isinstance(middleware_setting, tuple):
                new_middleware_setting = middleware_setting + (middleware_class_name,)
            else:
                err_msg = "{} is missing from {}.".format(
                    middleware_class_name, setting_name
                )
                raise TypeError(err_msg)

            setattr(settings, setting_name, new_middleware_setting)

    @classmethod
    def capture_interaction(cls, request, response, cached_request_body):
        cls.get_interaction_container().capture_interaction(
            request, response, cached_request_body
        )

    @classmethod
    def set_up(cls):
        cls.add_middleware()
        cls.set_up_optic()
        cls.set_up_additional()

    @classmethod
    def set_up_additional(cls):
        cls.set_up_interaction_container()

    @classmethod
    def set_up_optic(cls):
        cls.optic = Optic(OpticDjangoAppConfig.get_optic_settings())

    @classmethod
    def set_up_interaction_container(cls):
        from optic_django_middleware.container import BasicInteractionContainer

        cls.interaction_container = BasicInteractionContainer()



# -*- coding: utf-8
import importlib
from dataclasses import dataclass, fields

from django.apps import AppConfig
from django.conf import settings
from optic import OpticConfig


@dataclass
class OpticDjangoConfig(OpticConfig):
    INTERACTION_MANAGER = "optic_django_middleware.manager.BasicOpticManager"


class OpticDjangoAppConfig(AppConfig):
    name = "optic_django_middleware"
    verbose_name = "Optic Middleware"

    setting_name = "OPTIC"

    default_settings = OpticDjangoConfig(framework="Django")

    @classmethod
    def get_setting(cls, setting_name):
        return getattr(settings, cls.setting_name, {}).get(
            setting_name, getattr(cls.default_settings, setting_name)
        )

    @classmethod
    def enabled(cls) -> bool:
        return cls.get_setting("ENABLE")

    @classmethod
    def log_path(cls) -> str:
        return cls.get_setting("LOG_PATH")

    def ready(self):
        if self.enabled():
            OpticDjangoAppConfig.get_manager().set_up()

    @classmethod
    def get_manager(cls):
        module_name, class_name = OpticDjangoAppConfig.get_setting(
            "INTERACTION_MANAGER"
        ).rsplit(".", 1)
        InteractionManager = getattr(importlib.import_module(module_name), class_name)
        return InteractionManager

    @classmethod
    def get_optic_settings(cls) -> OpticConfig:
        field_names = set(f.name for f in fields(OpticConfig))
        c = {k: cls.get_setting(k) for k in field_names}
        return OpticConfig(**c)



import json
from typing import Dict
from urllib.parse import urlparse

from django.http import HttpRequest, HttpResponse
from kubi_ecs_logger import Logger
from kubi_ecs_logger.models import BaseSchema


class OpticEcsLogger(Logger):
    def __init__(self, *args, **kwargs):
        super(OpticEcsLogger, self).__init__(*args, **kwargs)

    def get_log_dict(self):
        return BaseSchema().dump(self._base)

    def serialize_to_ecs(
        self, request: HttpRequest, response: HttpResponse, request_body: bytes
    ) -> Dict:
        parsed_url = urlparse(request.build_absolute_uri())
        # Todo Add support for optional django-user-agents package
        user_agent_string = getattr(
            request.headers,
            "user_agent",
            None,
        )
        serialized_request_body: str
        if request.method == "GET":
            serialized_request_body = json.dumps(dict(request.GET))
        else:
            # For any other methods like POST, PUT, PATH
            if request.headers.get("content-type") == "application/json":
                serialized_request_body = request_body.decode("utf-8")
            else:
                serialized_request_body = json.dumps(dict(request.POST))

        if not user_agent_string and "HTTP_USER_AGENT" in request.META:
            user_agent_string = request.META["HTTP_USER_AGENT"]

        log = (
            self.url(
                full=parsed_url.geturl(),
                path=parsed_url.path,
                domain=parsed_url.hostname,
                port=parsed_url.port,
                query=parsed_url.query,
            )
            .http_response(
                status_code=response.status_code,
                body_content=response.content,
                bytes=len(response.content),
            )
            .http_request(
                body_content=serialized_request_body,
                method=request.method,
                bytes=len(serialized_request_body),
            )
            .user_agent(original=user_agent_string)
        )
        request_headers = {k: v for k, v in request.headers.items()}
        response_headers = {k: v for k, v in response.headers.items()}

        obj = log.get_log_dict()
        obj["http"] = {}
        if "httpresponse" in obj:
            obj["http"]["response"] = {
                "body": {
                    "content": obj["httpresponse"]["body_content"],
                    "bytes": obj["httpresponse"]["bytes"],
                },
                "status_code": int(obj["httpresponse"]["status_code"]),
                "headers": response_headers,
            }
        if "httprequest" in obj:
            obj["http"]["request"] = {
                "body": {
                    "content": obj["httprequest"]["body_content"],
                    "bytes": obj["httprequest"]["bytes"],
                },
                "method": obj["httprequest"]["method"],
                "headers": request_headers,
            }
        del obj["httpresponse"]
        del obj["httprequest"]
        return obj
